-- Copyright (C) by Jiancong Ni

local ffi = require "ffi"
local string_format = string.format
local ffi_new = ffi.new
local ffi_str = ffi.string
local C = ffi.C
local setmetatable = setmetatable
local error = error
local type = type

ffi.cdef[[
	typedef struct zlog_category_s zlog_category_t;
	int zlog_init(const char *confpath);
	int dzlog_init(const char *confpath, const char *cname);
	int zlog_reload(const char *confpath);
	void zlog_fini(void);
	zlog_category_t *zlog_get_category(const char *cname);
	void zlog(zlog_category_t * category,
	const char *file, size_t filelen,
	const char *func, size_t funclen,
	long line, int level,
	const char *format, ...) __attribute__((format(printf,8,9)));
	void dzlog(const char *file, size_t filelen,
	const char *func, size_t funclen,
	long line, int level,
	const char *format, ...) __attribute__((format(printf,7,8)));
]]

local so_file = "/soooner/soooner_report/lib/libzlog.so"
local libzlog = ffi.load(so_file)

local _M = { _VERSION = '0.01' }
local mt = { __index = _M }
local init_ok
local levels = {
	debug = 20,
	info = 40,
	notice = 60,
	warn = 80,
	error = 100,
	fatal = 120
}

function _M.init(etc_file)
	if init_ok ~= 0 then
		init_ok = libzlog.zlog_init(etc_file)
		if init_ok ~= 0 then
			return nil, "long conf error"
		end
	end
	return true
end
function _M.reload(etc_file)
	if init_ok == 0 then
		local ok = libzlog.zlog_reload(etc_file)
		if ok ~= 0 then
			return nil, "reload error"
		end
	end
	return true
end

function _M.new(self,category)
	if init_ok ~= 0 then return nil, "do not init" end
	local cat = libzlog.zlog_get_category(category)
	if not cat then
		return nil, "get_category error"
	end
	return setmetatable({ cat = cat }, mt)
end

for _k, _v in pairs(levels) do
	_M[_k] =
	function(self, fmt, ...)
		if init_ok == 0 then
			libzlog.zlog(self.cat, "", 0, "", 0, 0, _v, string_format(fmt, ...))
		end
	end
end

return _M

